@@ -1,7 +1,7 @@
 use egui_backend::{
-    egui::{vec2, Color32, Image},
+    egui::{Color32, Image},
     fltk::{enums::*, prelude::*, *},
-    gl, DpiScaling,
+    glow, tex_handle_from_vec_color32,
 };
 
 use fltk_egui as egui_backend;
@@ -16,23 +16,25 @@ const PIC_WIDTH: i32 = 320;
 const PIC_HEIGHT: i32 = 192;
 
 fn main() {
-    let a = app::App::default();
-    let mut win = window::GlWindow::new(100, 100, SCREEN_WIDTH as _, SCREEN_HEIGHT as _, None);
-    win.set_mode(Mode::Opengl3);
+    let fltk_app = app::App::default();
+    let mut win = window::GlWindow::new(100, 100, SCREEN_WIDTH as _, SCREEN_HEIGHT as _, None)
+        .center_screen();
+    win.set_mode(Mode::MultiSample);
     win.end();
     win.make_resizable(true);
     win.show();
     win.make_current();
 
-    let (painter, egui_input_state) = egui_backend::with_fltk(&mut win, DpiScaling::Default);
-    let mut egui_ctx = egui::CtxRef::default();
+    //Init backend
+    let (gl, mut painter, egui_state) = egui_backend::with_fltk(&mut win);
 
-    let state = Rc::from(RefCell::from(egui_input_state));
-    let painter = Rc::from(RefCell::from(painter));
+    //Init egui ctx
+    let egui_ctx = egui::Context::default();
+
+    let state = Rc::from(RefCell::from(egui_state));
 
     win.handle({
         let state = state.clone();
-        let painter = painter.clone();
         move |win, ev| match ev {
             enums::Event::Push
             | enums::Event::Released
@@ -42,36 +44,22 @@ fn main() {
             | enums::Event::Resize
             | enums::Event::Move
             | enums::Event::Drag => {
-                let mut state = state.borrow_mut();
-                state.fuse_input(win, ev, &mut painter.borrow_mut());
-                true
+                // Using "if let ..." for safety.
+                if let Ok(mut state) = state.try_borrow_mut() {
+                    state.fuse_input(win, ev);
+                    true
+                } else {
+                    false
+                }
             }
             _ => false,
         }
     });
 
     let start_time = Instant::now();
-    let mut srgba: Vec<Color32> = Vec::new();
 
-    //For now we will just set everything to black, because
-    //we will be updating it dynamically later. However, this could just as
-    //easily have been some actual picture data loaded in.
-    for _ in 0..PIC_HEIGHT {
-        for _ in 0..PIC_WIDTH {
-            srgba.push(Color32::BLACK);
-        }
-    }
-
-    //The user texture is what allows us to mix Egui and GL rendering contexts.
-    //Egui just needs the texture id, as the actual texture is managed by the backend.
-    let chip8_tex_id = painter.borrow_mut().new_user_texture(
-        (PIC_WIDTH as usize, PIC_HEIGHT as usize),
-        &srgba,
-        false,
-    );
-
-    //We will draw a crisp white triangle using OpenGL.
-    let triangle = triangle::Triangle::new();
+    //We will draw a crisp white triangle using Glow OpenGL.
+    let triangle = triangle::Triangle::new(&gl);
 
     //Some variables to help draw a sine wave
     let mut sine_shift = 0f32;
@@ -80,26 +68,20 @@ fn main() {
         "A text box to write in. Cut, copy, paste commands are available.".to_owned();
     let mut quit = false;
 
-    while a.wait() {
+    while fltk_app.wait() {
+        // Clear the screen to dark red
+        draw_background(&gl);
+
         let mut state = state.borrow_mut();
-        let mut painter = painter.borrow_mut();
         state.input.time = Some(start_time.elapsed().as_secs_f64());
-        let (egui_output, paint_cmds) = egui_ctx.run(state.input.take(), |ctx| {
-            //An example of how OpenGL can be used to draw custom stuff with egui
-            //overlaying it:
-            //First clear the background to something nice.
-            unsafe {
-                // Clear the screen to black
-                gl::ClearColor(0.3, 0.6, 0.3, 1.0);
-                gl::Clear(gl::COLOR_BUFFER_BIT);
-            }
+        let egui_output = egui_ctx.run(state.take_input(), |ctx| {
 
             //Then draw our triangle.
-            triangle.draw();
+            triangle.draw(&gl);
 
+            //Draw a cool sine wave in a buffer.
             let mut srgba: Vec<Color32> = Vec::new();
             let mut angle = 0f32;
-            //Draw a cool sine wave in a buffer.
             for y in 0..PIC_HEIGHT {
                 for x in 0..PIC_WIDTH {
                     srgba.push(Color32::BLACK);
@@ -111,16 +93,14 @@ fn main() {
                     }
                 }
             }
+
             sine_shift += 0.1f32;
 
-            //This updates the previously initialized texture with new data.
-            //If we weren't updating the texture, this call wouldn't be required.
-            painter.update_user_texture_data(chip8_tex_id, &srgba);
 
             egui::Window::new("Egui with FLTK and GL").show(&ctx, |ui| {
                 //Image just needs a texture id reference, so we just pass it the texture id that was returned to us
-                //when we previously initialized the texture.
-                ui.add(Image::new(chip8_tex_id, vec2(PIC_WIDTH as f32, PIC_HEIGHT as f32)));
+                let texture = tex_handle_from_vec_color32(ctx, "sinewave",srgba, [PIC_WIDTH as usize,PIC_HEIGHT as usize]);
+                ui.add(Image::new(texture.id(), texture.size_vec2()));
                 ui.separator();
                 ui.label("A simple sine wave plotted onto a GL texture then blitted to an egui managed Image.");
                 ui.label(" ");
@@ -135,15 +115,17 @@ fn main() {
             });
         });
 
-        state.fuse_output(&mut win, &egui_output);
+        state.fuse_output(&mut win, egui_output.platform_output);
 
-        let paint_jobs = egui_ctx.tessellate(paint_cmds);
+        let meshes = egui_ctx.tessellate(egui_output.shapes);
 
-        //Note: passing a bg_color to paint_jobs will clear any previously drawn stuff.
-        //Use this only if egui is being used for all drawing and you aren't mixing your own Open GL
-        //drawing calls with it.
-        //Since we are custom drawing an OpenGL Triangle we don't need egui to clear the background.
-        painter.paint_jobs(None, paint_jobs, &egui_ctx.font_image());
+        painter.paint_and_update_textures(
+            &gl,
+            state.canvas_size,
+            state.pixels_per_point(),
+            meshes,
+            &egui_output.textures_delta,
+        );
 
         win.swap_buffers();
         win.flush();
@@ -153,4 +135,13 @@ fn main() {
             break;
         }
     }
+
+    triangle.free(&gl);
+}
+
+fn draw_background<GL: glow::HasContext>(gl: &GL) {
+    unsafe {
+        gl.clear_color(0.6, 0.3, 0.3, 1.0);
+        gl.clear(glow::COLOR_BUFFER_BIT);
+    }
 }
